#!/usr/bin/perl

# Grabber for overwriting data by WeatherUnderground data

# Copyright 2016-2019 Michael Schlenstedt, michael@loxberry.de
#                       Christian Fenzl, christian@loxberry.de
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

use strict;
use warnings;

##########################################################################
# procedure - modified for FOSHKplugin by Oliver Engel
# 1. copy current.dat to current.dat.tmp
# 2. read current.dat.tmp into array A
# 3. read http-WU-message into array B
# 4. exchange all filled fields from 2 (B) into 1 (A)
# 5. write array A to current.dat.tmp on disk
# 6. copy current.dat.tmp --> current.dat
##########################################################################

##########################################################################
# Modules
##########################################################################

use LoxBerry::System;
use LoxBerry::Log;
use LWP::UserAgent;
use JSON qw( decode_json );
use File::Copy;
use Getopt::Long;
use Time::Piece;
#use Data::Dumper;
use Encode;
use POSIX qw(strftime);

sub windDirText {
  my ($grad) = $_[0];
  my ($lang) = $_[1];
  my $zeile = 0;
  my @arr=(
        ["N","NNE","NE","ENE","E","ESE", "SE", "SSE","S","SSW","SW","WSW","W","WNW","NW","NNW"],
        ["Nord","Nordnordost","Nordost","Ostnordost","Ost","Ostsüdost","Südost","Südsüdost","Süd","Südsüdwest","Südwest","Westsüdwest","West","Westnordwest","Nordwest","Nordnordwest"],
        ["noord","noordnoordoost","noordoost","oostnoordoost","oost","oostzuidoost","zuidoost","zuidzuidoost","zuid","zuidzuidwest","zuidwest","westzuidwest","west","westnoordwest","noordwest","noordnoordwest"],
        ["norte","norte-noreste","noreste","este-noreste","este","este-sureste", "sureste", "sur-sureste","sur","sur-suroeste","suroeste","oeste-suroeste","oeste","oeste-noroeste","noroeste","norte-noroeste"],
        ["north","north-northeast","northeast","east-northeast","east","east-southeast","southeast","south-southeast","south","south-southwest","southwest","west-southwest","west","west-northwest","north-west","north-northwest"]
	);
  if ($lang eq "ZZ") { $zeile = 0; }
  elsif ($lang eq "DE") { $zeile = 1; }
  elsif ($lang eq "NL") { $zeile = 2; }
  elsif ($lang eq "ES") { $zeile = 3; }
  else { $zeile = 4; }
  return $arr[$zeile][int(((($grad + 11.25) % 360) / 22.5)+0.5)];
}

##########################################################################
# Read Settings
##########################################################################

# Version of this script
my $version = "0.0.0.1";

my $pcfg               = new Config::Simple("$lbpconfigdir/weather4lox.cfg");
my $wuurl              = $pcfg->param("WULOCAL.URL");
my $stationid          = $pcfg->param("WULOCAL.STATIONID");
my $apikey             = $pcfg->param("WULOCAL.APIKEY");
my $currentnametmp     = "$lbplogdir/current.dat.tmp";
my $currentname        = "$lbplogdir/current.dat";
my $lblanguage         = LoxBerry::System::lblanguage;

if (not defined $wuurl) {
        LOGCRIT "You have to specify the URL of local weather station in WULOCAL\\URL in config-file";
        exit 2;
}
if ($stationid eq "") { $stationid = "FOSHKplugin"; }
if ($apikey eq "") { $apikey = "none"; }

# Read language phrases
my %L = LoxBerry::System::readlanguage("language.ini");

# Create a logging object
my $log = LoxBerry::Log->new (
        package => 'weather4lox',
        name => 'grabber_wu-local',
        logdir => "$lbplogdir",
);

# Commandline options
my $verbose = '';

GetOptions ('verbose' => \$verbose,
            'quiet'   => sub { $verbose = 0 });

# Due to a bug in the Logging routine, set the loglevel fix to 3
if ($verbose) {
        $log->stdout(1);
        $log->loglevel(7);
}

LOGSTART "Weather4Lox GRABBER_WUNDERGROUND-LOCAL process started";
LOGDEB "This is $0 Version $version";

my $urlstatus;
my $urlstatuscode;
my $ua;
my $resp;
# Get data from local Wunderground Server (API request) for current conditions
my $wgqueryurlcr = "$wuurl?apiKey=$apikey&stationId=$stationid&format=json&units=m";

LOGINF "Fetching Data for local Station $stationid";
LOGDEB "URL: $wgqueryurlcr";

$ua = new LWP::UserAgent;
$resp = $ua->get($wgqueryurlcr);
my $json = $resp->decoded_content();
$json = encode('utf8', decode('latin1',$json));

# Check status of request
$urlstatus = $resp->status_line;
$urlstatuscode = substr($urlstatus,0,3);

LOGDEB "Status: $urlstatus";

if ($urlstatuscode ne "200") {
  LOGCRIT "Failed to fetch data for $stationid\. Status Code: $urlstatuscode";
  exit 2;
} else {
  LOGOK "Data fetched successfully for $stationid";
}

# Decode JSON response from server
my $decoded_json = decode_json( $json );
#print Dumper $decoded_json;

# Write location data into database
my $t = localtime($decoded_json->{observations}->[0]->{epoch});
LOGINF "Saving new Data for Timestamp $t to database.";

my %wu_weather;
my @wu_weather_arr;
my %wu_response;

# ColNr beginning with 0
# See data/current.format
%wu_weather = (
        "cur_epoch" => 0,
        "cur_time" => 1,
        "cur_tt" => 11,
        "cur_tt_fl" => 12,
        "cur_hu" => 13,
        "cur_w_dir_txt" => 14,
        "cur_w_dir" => 15,
        "cur_w_sp" => 16,
        "cur_w_gu" => 17,
        "cur_w_ch" => 18,
        "cur_pr" => 19,
        "cur_dp" => 20,
        "cur_sr" => 22,
        "cur_hi" => 23,
        "cur_uvi" => 24,
        "cur_prec_today" => 25,
        "cur_prec_1hr" => 26
);

# Generate array from hash
@wu_weather_arr = ( keys %wu_weather );

LOGDEB "Data to request: " . join(', ', @wu_weather_arr);

# Convert anglo JSON data from WU to metric
$wu_response{cur_epoch} = $decoded_json->{observations}->[0]->{epoch} if ($decoded_json->{observations}->[0]->{epoch} ne "null");
$wu_response{cur_time} = strftime("%a, %d %b %Y %H:%M:%S %z", localtime($wu_response{cur_epoch}));
$wu_response{cur_tt} = sprintf("%.1f",$decoded_json->{observations}->[0]->{metric}->{temp}) if ($decoded_json->{observations}->[0]->{metric}->{temp} ne "-9999");
$wu_response{cur_tt_fl} = sprintf("%.1f",$decoded_json->{observations}->[0]->{metric}->{windChill}) if ($decoded_json->{observations}->[0]->{metric}->{windChill} ne "-9999");
$wu_response{cur_hu} = $decoded_json->{observations}->[0]->{humidity} if ($decoded_json->{observations}->[0]->{humidity} ne "-9999");
$wu_response{cur_w_dir} = $decoded_json->{observations}->[0]->{winddir} if ($decoded_json->{observations}->[0]->{winddir} ne "-9999");
$wu_response{cur_w_dir_txt} = windDirText($wu_response{cur_w_dir}, $lblanguage);
$wu_response{cur_w_sp} = $decoded_json->{observations}->[0]->{metric}->{windSpeed} if ($decoded_json->{observations}->[0]->{metric}->{windSpeed} ne "-9999");
$wu_response{cur_w_gu} = $decoded_json->{observations}->[0]->{metric}->{windGust} if ($decoded_json->{observations}->[0]->{metric}->{windGust} ne "-9999");
$wu_response{cur_w_ch} = sprintf("%.1f",$decoded_json->{observations}->[0]->{metric}->{windChill}) if ($decoded_json->{observations}->[0]->{metric}->{windChill} ne "-9999");
$wu_response{cur_pr} = $decoded_json->{observations}->[0]->{metric}->{pressure} if ($decoded_json->{observations}->[0]->{metric}->{pressure} ne "-9999");
$wu_response{cur_dp} = $decoded_json->{observations}->[0]->{metric}->{dewpt} if ($decoded_json->{observations}->[0]->{metric}->{dewpt} ne "-9999");
$wu_response{cur_sr} = sprintf("%.0f",$decoded_json->{observations}->[0]->{solarRadiation}) if ($decoded_json->{observations}->[0]->{solarRadiation} ne "-9999");
$wu_response{cur_hi} = sprintf("%.1f",$decoded_json->{observations}->[0]->{metric}->{heatIndex}) if ($decoded_json->{observations}->[0]->{metric}->{heatIndex} ne "-9999");
$wu_response{cur_uvi} = $decoded_json->{observations}->[0]->{uv} if ($decoded_json->{observations}->[0]->{uv} ne "-9999");
$wu_response{cur_prec_today} = $decoded_json->{observations}->[0]->{metric}->{precipTotal} if ($decoded_json->{observations}->[0]->{metric}->{precipTotal} ne "-9999");
$wu_response{cur_prec_1hr} = $decoded_json->{observations}->[0]->{metric}->{precipRate} if ($decoded_json->{observations}->[0]->{metric}->{precipRate} ne "-9999");

LOGDEB "Copying current.dat to current.dat.tmp";
copy($currentname, $currentnametmp);

LOGINF "Reading current.dat.tmp";

my $datafile_str = LoxBerry::System::read_file($currentnametmp);
chomp($datafile_str);

LOGDEB "Old line: $datafile_str";

my @values = split /\|/, $datafile_str;

foreach my $resp (keys %wu_weather ) {
        #print STDERR "Object $resp has value " . $wu_response{$resp} . "\n";
        if($wu_response{$resp} and $wu_response{$resp} ne "" and $wu_response{$resp} ne "-9999") {
                my $col = $wu_weather{$resp};
                $values[$col] = $wu_response{$resp};
                $values[$col] =~ s/^([-\d\.]+).*/$1/g;
                LOGDEB "  Response from $resp (value $values[$col]) is set to column $col";
        }
}

# Joining line
my $newline = join('|', @values);

LOGDEB "New line: $newline";

# Write patched file
eval {
        open(my $fh, ">$currentnametmp");
        #binmode $fh, ':encoding(UTF-8)';
        print $fh $newline;
        close $fh;
}
or do {
    LOGCRIT "Could not write $currentnametmp: $@";
        exit 2;
};

# Test file
my $currentsize = -s ($currentnametmp);
if ($currentsize > 100) {
        move($currentnametmp, $currentname);
} else {
        LOGCRIT "File size below 100 bytes - no new file created: $currentnametmp";
        exit 2;
}

# Give OK status to client.
LOGOK "Current Data saved successfully.";

# Exit
exit;

END
{
        LOGEND;
}

